#!/usr/bin/python3
#coding: utf-8
import subprocess, re, configparser, sys, os, time
from smtplib import SMTP_SSL, SMTP
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
import logging


class CephMonitor:
    dirpath = os.path.dirname(__file__)
    config_file = os.path.join(dirpath, 'config.ini')
    logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

    def get_ceph_status(self):
        p = subprocess.Popen(['timeout','60','ceph','-s'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, error = p.communicate()
        str_out = out.decode('utf-8')
        if not error.decode('utf-8'):
            status = re.search('health: (.+)', str_out)
            if status is not None:
                if status.group(1) != 'HEALTH_OK':
                    start = str_out.find('health:')
                    end = str_out.find('services:')
                    warn_msg = '\n'.join([l.strip() for l in str_out[start:end].splitlines() if l.strip()])
                    logging.error(warn_msg)
                else:
                    logging.info('ceph健康状态正常.')
                    warn_msg = None
            else:
                warn_msg = '无法获取ceph集群状态.'
                logging.error(warn_msg)

        else:
            warn_msg = '无法通过ceph命令获取集群状态.'
            logging.error(warn_msg)
        return warn_msg

    def send_mail(self, warn_msg=None):
        if warn_msg is not None:
            config = configparser.ConfigParser()
            config.read(self.config_file)
            smtp = config.get('email', 'smtp')
            port = int(config.get('email', 'port'))
            send_user = config.get('email', 'sned_user')
            password = config.get('email', 'password')
            recv_user = config.get('email', 'recv_user')
            ssl = config.get('email', 'ssl')
            if ssl == 'true':
                smtp_server = SMTP_SSL(smtp, port)
            else:
                smtp_server = SMTP(smtp, port)
            message = MIMEMultipart()
            message["From"] = send_user
            message["To"] = recv_user
            message["Subject"] = 'Ceph存储异常告警'
            message.attach(MIMEText(warn_msg, "plain"))
            try:
                smtp_server.login(send_user, password)
                smtp_server.sendmail(send_user, recv_user.split(','), message.as_string())
            except Exception as e:
                logging.error(f'邮件发送失败,请检查邮箱配置,错误信息:{e}')
def main():
    mon = CephMonitor()
    if len(sys.argv) > 1:
        mon.send_mail("这是一封测试邮件")
        sys.exit(0)
    count = 0
    while True:
        msg = mon.get_ceph_status()
        if msg is not None:
            count += 1
        else:
            count = 0

        if count >= 3:
            mon.send_mail(msg)
            count = 0
        time.sleep(60)

if __name__ == '__main__':
    main()
